// Package models
// @Author gmy 2024/8/7 9:38:00
package models

import (
	"errors"
	"github.com/beego/beego/v2/client/orm"
)

type BaseFromLyIndexRecord struct {
	BaseFromLyIndexRecordId int    `orm:"column(base_from_ly_index_record_id);pk"` // 指标记录ID
	CreateTime              string `orm:"column(create_time)"`                     // 创建时间
	ModifyTime              string `orm:"column(modify_time)"`                     // 修改时间
	Product                 string `orm:"column(product)"`                         // 产品
	Category                string `orm:"column(category)"`                        // 分类
	Url                     string `orm:"column(url)"`                             // 指标页面地址
	DataTime                string `orm:"column(data_time)"`                       // 数据日期
}

// 在 init 函数中注册模型
func init() {
	orm.RegisterModel(new(BaseFromLyIndexRecord))
}

// AddLyIndexRecord 添加指标记录
func AddLyIndexRecord(item *BaseFromLyIndexRecord) (int64, error) {
	o := orm.NewOrm()
	id, err := o.Insert(item)
	if err != nil {
		return 0, err
	}
	return id, nil
}

// GetLyIndexRecordByUrl 查询指标记录是否存在
func GetLyIndexRecordByUrl(url string) (item *BaseFromLyIndexRecord, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_ly_index_record WHERE url=?`
	err = o.Raw(sql, url).QueryRow(&item)

	if errors.Is(err, orm.ErrNoRows) {
		return nil, nil
	}

	return
}

func GetLyIndexRecordByCondition(condition string, pars []interface{}) (count int, err error) {
	o := orm.NewOrm()
	sql := `SELECT count(1) FROM base_from_ly_index_record WHERE 1 = 1 `
	sql += condition
	err = o.Raw(sql, pars...).QueryRow(&count)
	if err != nil {
		return 0, err
	}
	return count, err
}
